@@ -1,8 +1,8 @@
 package Search::OpenSearch::Engine::Lucy;
-use strict;
-use warnings;
+use Moose;
 use Carp;
-use base qw( Search::OpenSearch::Engine );
+extends 'Search::OpenSearch::Engine';
+use Types::Standard qw( Bool Str );
 use SWISH::Prog::Lucy::Indexer;
 use SWISH::Prog::Lucy::Searcher;
 use SWISH::Prog::Doc;
@@ -10,38 +10,29 @@ use Lucy::Object::BitVector;
 use Lucy::Search::Collector::BitCollector;
 use Data::Dump qw( dump );
 use Scalar::Util qw( blessed );
-use Module::Load;
+use Class::Load;
 use Path::Class::Dir;
 use SWISH::3 qw(:constants);
 use Search::Tools;
+use Try::Tiny;
 
-our $VERSION = '0.19';
+our $VERSION = '0.300';
 
-__PACKAGE__->mk_accessors(
-    qw(
-        aggregator_class
-        auto_commit
-        )
-);
-
-use Rose::Object::MakeMethods::Generic ( 'scalar --get_set_init' => 'indexer',
-);
+has 'aggregator_class' =>
+    ( is => 'rw', isa => Str, default => sub {'SWISH::Prog::Aggregator'} );
+has 'auto_commit' => ( is => 'rw', isa => Bool, default => sub {1} );
 
 sub type {'Lucy'}
 
-sub init {
+sub BUILD {
     my $self = shift;
-    $self->SUPER::init(@_);
-    $self->{aggregator_class} ||= 'SWISH::Prog::Aggregator';
-    $self->{array_field_values} = 1;
-    $self->{auto_commit} = 1 unless defined $self->{auto_commit};
-    load $self->{aggregator_class};
+    Class::Load::load_class( $self->aggregator_class );
     return $self;
 }
 
 sub init_searcher {
     my $self     = shift;
-    my $index    = $self->index or croak "index not defined";
+    my $index    = $self->index or confess "index not defined";
     my $searcher = SWISH::Prog::Lucy::Searcher->new(
         invindex => [@$index],      # copy so that suggester can use strings
         debug    => $self->debug,
@@ -62,19 +53,22 @@ sub init_suggester {
 
     # Text::Aspell is optional, so verify we have it
     # before claiming to have a Suggester.
-    eval {
+    my $has_suggester = try {
         require LucyX::Suggester;
         $conf{spellcheck} = Search::Tools->spellcheck(%$spellcheck_conf);
         if ( $ENV{TEST_SPELLCHECK_MISSING} ) {
             die "testing missing spellcheck";
         }
-    };
-    if ($@) {
-        if ( $self->debug and $self->logger ) {
-            $self->logger->log("Failed to load LucyX::Suggester: $@");
-        }
-        return;
+        return 1;
     }
+    catch {
+        if ( $self->debug and $self->logger ) {
+            $self->logger->log("Failed to load LucyX::Suggester: $_");
+        }
+        return 0;
+    };
+    return unless $has_suggester;
+
     my $suggester = LucyX::Suggester->new(
         indexes => $self->index,
         debug   => $self->debug,
@@ -84,8 +78,8 @@ sub init_suggester {
 }
 
 sub build_facets {
-    my $self    = shift;
-    my $query   = shift;
+    my $self  = shift;
+    my $query = shift;
     confess "query required" unless defined $query;
     my $results = shift or croak "results required";
     if ( $self->debug and $self->logger ) {
@@ -211,13 +205,22 @@ sub _massage_rest_req_into_doc {
 
 sub init_indexer {
     my $self = shift;
+    my $idx = shift || 0;
+
+    if ( $idx =~ m/\D/ ) {
+        confess "idx must be an integer for reading into array of index()";
+    }
+    if ( $idx > scalar @{ $self->index } ) {
+        confess sprintf( "idx %d > than index array size %d",
+            $idx, scalar @{ $self->index } );
+    }
 
     # unlike a Searcher, which has an array of invindex objects,
     # the Indexer wants only one. We take the first by default,
     # but a subclass could do more subtle logic here.
 
     my $indexer = SWISH::Prog::Lucy::Indexer->new(
-        invindex => $self->index->[0],
+        invindex => $self->index->[$idx],
         debug    => $self->debug,
         %{ $self->indexer_config },
     );
@@ -225,6 +228,7 @@ sub init_indexer {
 }
 
 # PUT only if it does not yet exist
+# note PUT operates only on first index if there are multiple.
 sub PUT {
     my $self = shift;
     my $req  = shift or croak "request required";
@@ -263,16 +267,31 @@ sub PUT {
     return { code => 201, total => $total, doc => $exists->{doc} };
 }
 
-# POST allows new and updates
-sub POST {
+sub _get_indexer {
     my $self = shift;
-    my $req  = shift or croak "request required";
-    my $doc  = $self->_massage_rest_req_into_doc($req);
-    my $uri  = $doc->url;
-    my $indexer
-        = $self->auto_commit
-        ? $self->init_indexer()
-        : $self->indexer();
+
+    # autocommit means we must manage our own indexer
+    # since we want to invalidate and re-create
+
+    if ( $self->auto_commit ) {
+        return $self->init_indexer(@_);
+    }
+
+    # did we have an indexer and it was invalidated? get new one.
+    if ( !$self->indexer ) {
+        $self->indexer( $self->init_indexer(@_) );
+    }
+    return $self->indexer;
+}
+
+# POST allows new and updates
+# note POST operates only on first index if there are multiple
+sub POST {
+    my $self    = shift;
+    my $req     = shift or croak "request required";
+    my $doc     = $self->_massage_rest_req_into_doc($req);
+    my $uri     = $doc->url;
+    my $indexer = $self->_get_indexer;
     $indexer->process($doc);
 
     if ( !$self->auto_commit ) {
@@ -295,6 +314,12 @@ sub COMMIT {
         return { code => 400 };
     }
     my $indexer = $self->indexer();
+
+    # is it possible to get here? croak just in case.
+    if ( !$indexer ) {
+        confess "Can't call COMMIT on an undefined indexer";
+    }
+
     if ( my $total = $indexer->count() ) {
         $indexer->finish();
 
@@ -331,20 +356,20 @@ sub DELETE {
             msg  => "$uri cannot be deleted because it does not exist"
         };
     }
-    my $indexer
-        = $self->auto_commit
-        ? $self->init_indexer()
-        : $self->indexer;
-    $indexer->get_lucy->delete_by_term(
-        field => 'swishdocpath',
-        term  => $uri,
-    );
 
+    my $i = 0;
+    for my $idx ( @{ $self->index } ) {
+        my $indexer = $self->_get_indexer( $i++ );
+        $indexer->get_lucy->delete_by_term(
+            field => 'swishdocpath',
+            term  => $uri,
+        );
+        next unless $self->auto_commit;
+        $indexer->finish();
+    }
     if ( !$self->auto_commit ) {
         return { code => 202 };
     }
-
-    $indexer->finish();
     return { code => 200, };
 }
 
@@ -531,7 +556,7 @@ calls, until COMMIT or ROLLBACK is called.
 
 Default is true (on).
 
-=head2 init
+=head2 BUILD
 
 Overrides base method to load the I<aggregator_class> and other
 Engine-specific construction tasks.
@@ -571,12 +596,20 @@ names.
 
 =head2 PUT( I<doc> )
 
+Writes I<doc> to the first index defined. I<doc> must already exist.
+
 =head2 POST( I<doc> )
 
+Writes I<doc> to the first index defined. I<doc> may be new or already exist.
+
 =head2 DELETE( I<uri> )
 
+Deletes I<uri> from all indexes.
+
 =head2 GET( I<uri> )
 
+Fetches I<uri> from all indexes.
+
 =head2 COMMIT
 
 If auto_commit is false, use this method to conclude a transaction.
